from django.conf.urls import url
from notify import views as nf

urlpatterns = [
    url(r'^all/$', nf.notifications, name="all"),
    url(r'^api/update/$', nf.notification_update, name="update"),
    url(r'^mark/$', nf.mark, name='mark'),
    url(r'^mark-all/$', nf.mark_all, name='mark_all'),
    url(r'^delete/$', nf.delete, name='delete'),
]



from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.http import JsonResponse, HttpResponseBadRequest
from django.shortcuts import render, redirect
from django.template.loader import render_to_string
from django.utils.http import is_safe_url
from django.views.decorators.http import require_POST
from notify.models import Notification

# TODO: Convert function-based views to Class-based views.


def notification_redirect(request, ctx):
    """
    Helper to handle HTTP response after an action is performed on notification

    :param request: HTTP request context of the notification

    :param ctx: context to be returned when a AJAX call is made.

    :returns: Either JSON for AJAX or redirects to the calculated next page.
    """
    if request.is_ajax():
        return JsonResponse(ctx)
    else:
        next_page = request.POST.get('next', reverse('notifications:all'))
        if not ctx['success']:
            return HttpResponseBadRequest(ctx['msg'])
        if is_safe_url(next_page):
            return redirect(next_page)
        else:
            return redirect(reverse('notifications:all'))


@login_required
def notifications(request):
    """
    Returns a rendered page of all notifications for the logged-in user.
    Uses ``notification.nf_type`` as the template for individual notification.
    Each notification type is expected to have a render template
    at ``notifications/includes/NOTIFICATION_TYPE.html``.

    :param request: HTTP request context.

    :return: Rendered notification list page.
    """
    notification_list = request.user.notifications.active()
    return render(request, 'notifications/all.html',
                  {'notifications': notification_list})


@login_required
@require_POST
def mark(request):
    """
    Handles marking of individual notifications as read or unread.
    Takes ``notification id`` and mark ``action`` as POST data.

    :param request: HTTP request context.

    :returns: Response to mark action of supplied notification ID.
    """
    notification_id = request.POST.get('id', None)
    action = request.POST.get('action', None)
    success = True

    if notification_id:
        try:
            notification = Notification.objects.get(pk=notification_id,
                                                    recipient=request.user)
            if action == 'read':
                notification.mark_as_read()
                msg = "Marked as read"
            elif action == 'unread':
                notification.mark_as_unread()
                msg = "Marked as unread"
            else:
                success = False
                msg = "Invalid mark action."
        except Notification.DoesNotExist:
            success = False
            msg = "Notification does not exists."
    else:
        success = False
        msg = "Invalid Notification ID"

    ctx = {'msg': msg, 'success': success, 'action': action}

    return notification_redirect(request, ctx)


@login_required
@require_POST
def mark_all(request):
    """
    Marks notifications as either read or unread depending of POST parameters.
    Takes ``action`` as POST data, it can either be ``read`` or ``unread``.

    :param request: HTTP Request context.

    :return: Response to mark_all action.
    """
    action = request.POST.get('action', None)
    success = True

    if action == 'read':
        request.user.notifications.read_all()
        msg = "Marked all notifications as read"
    elif action == 'unread':
        request.user.notifications.unread_all()
        msg = "Marked all notifications as unread"
    else:
        msg = "Invalid mark action"
        success = False

    ctx = {'msg': msg, 'success': success, 'action': action}

    return notification_redirect(request, ctx)


@login_required
@require_POST
def delete(request):
    """
    Deletes notification of supplied notification ID.

    Depending on project settings, if ``NOTIFICATIONS_SOFT_DELETE``
    is set to ``False``, the notifications will be deleted from DB.
    If not, a soft delete will be performed.

    By default, notifications are deleted softly.

    :param request: HTTP request context.

    :return: Response to delete action on supplied notification ID.
    """
    notification_id = request.POST.get('id', None)
    success = True

    if notification_id:
        try:
            notification = Notification.objects.get(pk=notification_id,
                                                    recipient=request.user)
            soft_delete = getattr(settings, 'NOTIFY_SOFT_DELETE', True)
            if soft_delete:
                notification.deleted = True
                notification.save()
            else:
                notification.delete()
            msg = "Deleted notification successfully"
        except Notification.DoesNotExist:
            success = False
            msg = "Notification does not exists."
    else:
        success = False
        msg = "Invalid Notification ID"

    ctx = {'msg': msg, 'success': success, }

    return notification_redirect(request, ctx)


@login_required
def notification_update(request):
    """
    Handles live updating of notifications, follows ajax-polling approach.

    Read more: http://stackoverflow.com/a/12855533/4726598

    Required URL parameters: ``flag``.

    Explanation:

        - The ``flag`` parameter carries the last notification ID \
        received by the user's browser.

        - This ``flag`` is most likely to be generated by using \
        a simple JS/JQuery DOM. Just grab the first element of \
        the notification list.

            - The element will have a ``data-id`` attribute set to the \
            corresponding notification.
            - We'll use it's value as the flag parameter.

        - The view treats the ``last notification flag`` as a model \
        ```filter()`` and fetches all notifications greater than \
        the flag for the user.

        - Then the a JSON data is prepared with all necessary \
        details such as, ``verb``, ``actor``, ``target`` and their \
        URL etc. The foreignkey are serialized as their \
        default ``__str__`` value.

            - Everything will be HTML escaped by django's ``escape()``.

        - Since these notification sent will only serve temporarily \
        on the notification box and will be generated fresh \
        using a whole template, to avoid client-side notification \
        generation using the JSON data, the JSON data will also \
        contain a rendered HTML string so that you can easily \
        do a JQuery ``$yourNotificationBox.prepend()`` on the \
        rendered html string of the notification.

        - The template used is expected to be different than the \
        template used in full page notification as the css \
        and some other elements are highly likely to be \
        different than the full page notification list. \

        - The template used will be the ``notification type`` of the \
        notification suffixed ``_box.html``. So, if your \
        notification type is ``comment_reply``, the template \
        will be ``comment_reply_box.html``.

            - This template will be stored in ``notifications/includes/`` \
            of  your template directory.

            - That makes: ``notifications/includes/comment_reply_box.html``

        - The rest is self-explanatory.

    :param request: HTTP request context.

    :return: Notification updates (if any) in JSON format.
    """
    flag = request.GET.get('flag', None)
    success = True

    if flag.isdigit():
        last_notification = int(flag)
    else:
        last_notification = None

    if last_notification:
        try:
            notification = Notification.objects.get(pk=last_notification,
                                                    recipient=request.user)
            new_notifications = request.user.notifications.filter(
                id__gt=notification.id).active()

            if new_notifications:
                msg = "Notifications successfully retrieved."
                notification_list = [nf.as_json() for nf in new_notifications]
                for notification in notification_list:
                    templates = [
                        'notifications/includes/%s_box.html'
                        % notification['nf_type'],
                        'notifications/includes/default_box.html']
                    notification['html'] = render_to_string(templates,
                                                            notification)
            else:
                msg = "No new notifications."
                notification_list = []

            ctx = {
                "retrieved": len(new_notifications),
                "unread_count": request.user.notifications.unread().count(),
                "notifications": notification_list,
                "success": success,
                "msg": msg,
            }

            return JsonResponse(ctx)

        except Notification.DoesNotExist:
            success = False
            msg = "Invalid notification flag"
    else:
        success = False
        msg = "Notification flag not sent."

    ctx = {"success": success, "msg": msg}

    return JsonResponse(ctx)



from django.test import TestCase, override_settings
from django.contrib.auth import get_user_model
from notify.models import Notification
from notify.signals import notify
from django.utils import timezone
from django.core.urlresolvers import reverse
import json
from django.template import Template, Context

User = get_user_model()


class NotificationManagerTest(TestCase):

    def setUp(self):
        self.recipient = User.objects.create(username='recipient',
                                             email='recipient@test.com',
                                             password='pwd@recipient')
        self.actor = User.objects.create(username='actor',
                                         email='actor@test.com',
                                         password='pwd@actor')
        self.nf_count = 10

        self.assertEqual(self.recipient.id, 1)
        self.assertEqual(self.actor.id, 2)

        for x in range(self.nf_count):
            notify.send(User, recipient=self.recipient, actor=self.actor,
                        verb='followed you')

    def test_unread(self):
        self.assertEqual(Notification.objects.unread().count(), self.nf_count)
        nf = Notification.objects.filter(recipient=self.recipient).first()
        nf.mark_as_read()
        self.assertEqual(Notification.objects.unread().count(),
                         self.nf_count - 1)

        for n in Notification.objects.unread():
            self.assertFalse(n.read)

    def test_read(self):
        self.assertEqual(Notification.objects.unread().count(), self.nf_count)
        nf = Notification.objects.filter(recipient=self.recipient).first()
        nf.mark_as_read()
        self.assertEqual(Notification.objects.read().count(), 1)

        for n in Notification.objects.read():
            self.assertTrue(n.read)

    def test_read_all(self):
        self.assertEqual(Notification.objects.unread().count(), self.nf_count)
        recipient_nfs = Notification.objects.filter(recipient=self.recipient)
        recipient_nfs.read_all()

        self.assertEqual(Notification.objects.read().count(), self.nf_count)
        self.assertEqual(Notification.objects.unread().count(), 0)

    def test_unread_all(self):
        self.assertEqual(Notification.objects.unread().count(), self.nf_count)

        recipient_nfs = Notification.objects.filter(recipient=self.recipient)
        recipient_nfs.read_all()

        self.assertEqual(Notification.objects.read().count(), self.nf_count)
        self.assertEqual(Notification.objects.unread().count(), 0)

        recipient_nfs.unread_all()

        self.assertEqual(Notification.objects.read().count(), 0)
        self.assertEqual(Notification.objects.unread().count(), self.nf_count)

    def test_delete_all_soft(self):
        self.assertEqual(Notification.objects.unread().count(), self.nf_count)

        nf = Notification.objects.filter(recipient=self.recipient).first()
        # Mark single notification as read
        nf.mark_as_read()

        self.assertEqual(Notification.objects.read().count(), 1)
        self.assertEqual(Notification.objects.unread().count(),
                         self.nf_count - 1)

        self.assertEqual(Notification.objects.active().count(), self.nf_count)
        self.assertEqual(Notification.objects.deleted().count(), 0)

        Notification.objects.delete_all()
        # Mark all as deleted
        self.assertEqual(Notification.objects.read().count(), 0)
        self.assertEqual(Notification.objects.unread().count(), 0)
        self.assertEqual(Notification.objects.active().count(), 0)
        self.assertEqual(Notification.objects.deleted().count(), self.nf_count)

        Notification.objects.active_all()
        # Mark all as active
        self.assertEqual(Notification.objects.read().count(), 1)
        self.assertEqual(Notification.objects.unread().count(),
                         self.nf_count - 1)
        self.assertEqual(Notification.objects.active().count(), self.nf_count)
        self.assertEqual(Notification.objects.deleted().count(), 0)

    @override_settings(NOTIFY_SOFT_DELETE=False)
    def test_delete_all_hard(self):
        self.assertEqual(Notification.objects.active().count(), self.nf_count)

        # Hard delete all notifications.
        Notification.objects.delete_all()
        self.assertEqual(Notification.objects.active().count(), 0)
        self.assertEqual(Notification.objects.deleted().count(), 0)


class NotificationTest(TestCase):

    def setUp(self):
        self.no_of_users = 10

        users = []
        for i in range(self.no_of_users):
            users.append(
                User(username='user-%r' % i, password='pwd@user%r' % i,
                     email='user%r@test.com' % i))
        User.objects.bulk_create(users)

        self.actor = User.objects.create(username='actor',
                                         password='pwd@actor',
                                         email='actor@test.com')

        self.recipient = User.objects.get(username='user-0')
        self.recipient_list = User.objects.filter(
            username__startswith='u').order_by('id')

    def test_created_users(self):
        users = User.objects.all()
        self.assertEqual(users.count(), self.no_of_users + 1)
        self.assertEqual(self.recipient_list.count(), self.no_of_users)

    def test_single_user_notify(self):
        notify.send(User, recipient=self.recipient, actor=self.actor,
                    verb='poked you')
        notification = Notification.objects.get(pk=1)
        self.assertEqual(notification.recipient_id, self.recipient.id)
        timedelta = timezone.now() - notification.created
        self.assertLessEqual(timedelta.seconds, 60)

    def test_multiple_user_notify(self):
        notify.send(User, recipient_list=list(self.recipient_list),
                    actor=self.actor, verb='uploaded a new video')
        notifications = Notification.objects.filter(verb__startswith='u')
        self.assertEqual(notifications.count(), self.no_of_users)

        username_list = [u.username for u in self.recipient_list]

        for nf in notifications:
            self.assertIn(nf.recipient.username, username_list)
            timedelta = timezone.now() - nf.created
            self.assertLessEqual(timedelta.seconds, 60)


class NotificationViewTest(TestCase):

    def setUp(self):
        self.no_of_users = 10

        # Create recipients
        users = []
        for i in range(self.no_of_users):
            u = User(username='user-%r' % i, email='user%r@test.com' % i)
            u.set_password('pwd@user')
            users.append(u)
        User.objects.bulk_create(users)

        # Create actor
        actor = User.objects.create(username='actor',
                                    email='actor@test.com')
        actor.set_password('pwd@actor')
        actor.save()
        self.actor = User.objects.get(username="actor")

        self.recipient = User.objects.get(username='user-0')

        self.recipient_list = User.objects.filter(
            username__startswith='u').order_by('id')

        # Send notifications to all recipients.
        notify.send(User, recipient_list=list(self.recipient_list),
                    actor=self.actor, verb='wrote a new blog post.')

        # Send some more notifications to a specific user.
        self.recipient_nf_count = 11
        for i in range(10):
            notify.send(User, recipient=self.recipient, actor_text='You',
                        verb='reached level %r' % i)

        # Login this recipient
        self.assertTrue(self.client.login(username="user-0",
                                          password="pwd@user"))

    def test_created_users(self):
        users = User.objects.all()
        self.assertEqual(users.count(), self.no_of_users + 1)
        self.assertEqual(self.recipient_list.count(), self.no_of_users)
        self.assertEqual(self.recipient.id, 1)
        self.assertTrue(self.recipient.is_active)

    def test_notifications_view(self):
        response = self.client.get(reverse('notifications:all'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['notifications'].count(),
                         self.recipient.notifications.count())
        self.assertEqual(response.context['notifications'].count(),
                         self.recipient_nf_count)
        self.assertTemplateUsed(response, 'notifications/all.html')

    def test_mark_notification_view_non_ajax_without_next_parameter(self):
        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient_nf_count)

        first_nf = self.recipient.notifications.first()
        first_nf_id = first_nf.id
        self.assertFalse(first_nf.read)

        # Marking as read.
        response = self.client.post(reverse('notifications:mark'),
                                    {'action': 'read', 'id': first_nf_id})
        self.assertRedirects(response, reverse('notifications:all'))

        nf = Notification.objects.get(pk=first_nf_id)
        self.assertTrue(nf.read)

        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient_nf_count - 1)

        # Marking as unread
        response_2 = self.client.post(reverse('notifications:mark'),
                                      {'action': 'unread', 'id': first_nf_id})
        self.assertRedirects(response_2, reverse('notifications:all'))
        nf_2 = Notification.objects.get(pk=first_nf_id)
        self.assertFalse(nf_2.read)
        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient_nf_count)

    def test_mark_notification_view_non_ajax_with_next_parameter(self):
        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient_nf_count)

        first_nf = self.recipient.notifications.first()
        first_nf_id = first_nf.id
        self.assertFalse(first_nf.read)

        # Marking as read
        ctx = {'action': 'read', 'id': first_nf_id, 'next': '/notifications/'}
        response = self.client.post(reverse('notifications:mark'), ctx)
        self.assertRedirects(response, 'http://testserver/notifications/',
                             target_status_code=404, status_code=302)

        nf = Notification.objects.get(pk=first_nf_id)
        self.assertTrue(nf.read)

        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient_nf_count - 1)

        # Marking as unread
        ctx2 = {'action': 'unread', 'id': first_nf_id,
                'next': '/notifications/'}
        response_2 = self.client.post(reverse('notifications:mark'), ctx2)
        self.assertRedirects(response_2, 'http://testserver/notifications/',
                             target_status_code=404, status_code=302)
        nf_2 = Notification.objects.get(pk=first_nf_id)
        self.assertFalse(nf_2.read)
        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient_nf_count)

    def test_mark_notification_view_ajax(self):
        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient_nf_count)

        first_nf = self.recipient.notifications.first()
        first_nf_id = first_nf.id
        self.assertFalse(first_nf.read)

        # Marking as read
        ctx = {'action': 'read', 'id': first_nf_id}
        response = self.client.post(reverse('notifications:mark'), ctx,
                                    HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

        nf = Notification.objects.get(pk=first_nf_id)
        self.assertTrue(nf.read)

        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient_nf_count - 1)

        # Marking as unread
        ctx = {'action': 'unread', 'id': first_nf_id}
        response_2 = self.client.post(reverse('notifications:mark'), ctx,
                                      HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response_2.status_code, 200)
        nf_2 = Notification.objects.get(pk=first_nf_id)
        self.assertFalse(nf_2.read)
        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient_nf_count)

    def test_mark_notification_view_for_arbitrary_user(self):
        """
        Our purpose here is to check whether user A can mark user B's
        notification as read or unread.
        """
        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient_nf_count)
        self.assertNotEqual(self.recipient.id, 2)

        nf_of_other_user = Notification.objects.get(recipient_id=2)
        self.assertFalse(nf_of_other_user.read)

        ctx = {'action': 'read', 'id': nf_of_other_user.id}

        response = self.client.post(reverse('notifications:mark'), ctx)

        self.assertEqual(response.status_code, 400)

        nf_of_other_user = Notification.objects.get(recipient_id=2)
        self.assertFalse(nf_of_other_user.read)
        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient_nf_count)

    # I'll escape the with and without `next` parameter test as it is
    # handled by the same functions, it is redundant to write them.

    def test_mark_all_notification_view_non_ajax(self):
        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient_nf_count)

        # Mark all as read
        ctx = {'action': 'read'}
        response = self.client.post(reverse('notifications:mark_all'), ctx)
        self.assertRedirects(response, reverse('notifications:all'))

        self.assertEqual(self.recipient.notifications.unread().count(), 0)
        self.assertEqual(self.recipient.notifications.read().count(),
                         self.recipient.notifications.count())

        # Mark all as unread
        ctx2 = {'action': 'unread'}
        response2 = self.client.post(reverse('notifications:mark_all'), ctx2)
        self.assertRedirects(response2, reverse('notifications:all'))

        self.assertEqual(self.recipient.notifications.read().count(), 0)
        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient.notifications.count())

    def test_mark_all_notification_view_ajax(self):
        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient_nf_count)

        # Mark all as read
        ctx = {'action': 'read'}
        response = self.client.post(reverse('notifications:mark_all'), ctx,
                                    HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

        self.assertEqual(self.recipient.notifications.unread().count(), 0)
        self.assertEqual(self.recipient.notifications.read().count(),
                         self.recipient.notifications.count())

        # Mark all as unread
        ctx2 = {'action': 'unread'}
        response2 = self.client.post(reverse('notifications:mark_all'), ctx2,
                                     HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response2.status_code, 200)

        self.assertEqual(self.recipient.notifications.read().count(), 0)
        self.assertEqual(self.recipient.notifications.unread().count(),
                         self.recipient.notifications.count())

    def test_soft_delete_notification_view_non_ajax(self):
        self.assertEqual(self.recipient.notifications.active().count(),
                         self.recipient_nf_count)

        first_nf = self.recipient.notifications.first()
        first_nf_id = first_nf.id
        self.assertFalse(first_nf.deleted)

        # Soft delete notification
        ctx = {'id': first_nf_id}
        response = self.client.post(reverse('notifications:delete'), ctx)
        self.assertRedirects(response, reverse('notifications:all'))

        nf = Notification.objects.get(pk=first_nf_id)
        self.assertTrue(nf.deleted)

        self.assertEqual(self.recipient.notifications.active().count(),
                         self.recipient_nf_count - 1)

    def test_soft_delete_notification_view_ajax(self):
        self.assertEqual(self.recipient.notifications.active().count(),
                         self.recipient_nf_count)

        first_nf = self.recipient.notifications.first()
        first_nf_id = first_nf.id
        self.assertFalse(first_nf.deleted)

        # Soft delete notification
        ctx = {'id': first_nf_id}
        response = self.client.post(reverse('notifications:delete'), ctx,
                                    HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

        nf = Notification.objects.get(pk=first_nf_id)
        self.assertTrue(nf.deleted)

        self.assertEqual(self.recipient.notifications.active().count(),
                         self.recipient_nf_count - 1)

    @override_settings(NOTIFY_SOFT_DELETE=False)
    def test_hard_delete_notification_view(self):
        self.assertEqual(self.recipient.notifications.all().count(),
                         self.recipient_nf_count)

        first_nf = self.recipient.notifications.first()
        first_nf_id = first_nf.id

        # Hard delete notification
        ctx = {'id': first_nf_id}
        response = self.client.post(reverse('notifications:delete'), ctx)
        self.assertRedirects(response, reverse('notifications:all'))

        self.assertRaises(Notification.DoesNotExist,
                          Notification.objects.get,
                          pk=first_nf_id)

        self.assertEqual(self.recipient.notifications.all().count(),
                         self.recipient_nf_count - 1)

    def test_delete_notification_view_for_arbitrary_user(self):
        """
        Our purpose here is to check whether user A can mark user B's
        notification as read or unread.
        """
        self.assertNotEqual(self.recipient.id, 2)

        nf_of_other_user = Notification.objects.get(recipient_id=2)
        self.assertFalse(nf_of_other_user.deleted)

        ctx = {'id': nf_of_other_user.id}

        response = self.client.post(reverse('notifications:mark'), ctx)

        self.assertEqual(response.status_code, 400)

        nf_of_other_user = Notification.objects.get(recipient_id=2)
        self.assertFalse(nf_of_other_user.deleted)

        self.assertEqual(self.recipient.notifications.active().count(),
                         self.recipient_nf_count)

    def test_update_notifications_view(self):
        self.assertEqual(self.recipient.notifications.active().count(),
                         self.recipient_nf_count)
        nf = Notification.objects.get(pk=12)
        self.assertEqual(nf.recipient.username, 'user-0')

        url = "{}?flag=12".format(reverse('notifications:update'))

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        resp = json.loads(response.content.decode('utf-8'))

        self.assertTrue(resp['success'])
        self.assertEqual(resp['unread_count'],
                         self.recipient.notifications.unread().count())
        notifications = self.recipient.notifications.active().filter(
            id__gt=12)
        self.assertEqual(resp['retrieved'],
                         notifications.count())

        for json_nf in resp['notifications']:
            self.assertIn(json_nf['notification_id'],
                          [nf.id for nf in notifications])

    def test_update_view_with_wrong_flag_value(self):
        nf = Notification.objects.get(pk=3)
        self.assertNotEqual(nf.recipient.username, 'user-0')

        url = "{}?flag=this-wont-work".format(reverse('notifications:update'))
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        resp = json.loads(response.content.decode('utf-8'))

        self.assertFalse(resp['success'])

        # A notification that does not exists
        url2 = "{}?flag=123456789".format(reverse('notifications:update'))
        response2 = self.client.get(url2)

        self.assertEqual(response2.status_code, 200)
        resp2 = json.loads(response2.content.decode('utf-8'))

        self.assertFalse(resp2['success'])


class NotificationTemplateTagTest(TestCase):

    RENDER_TEMPLATE = Template("""
    {% load notification_tags %}
    {% render_notifications using notifications for page %}
    """)

    JS_INCLUSION = Template("""
    {% load notification_tags  %}
    {% include_notify_js_variables %}
    """)

    def setUp(self):
        user = User(username='user', email='user@test.com')
        user.set_password('pwd@user')
        user.save()

        self.user = User.objects.get(username='user')

    def test_render_template_tag(self):
        notify.send(User, recipient=self.user,
                    actor_text='Joe', verb='followed you')
        nf_list = Notification.objects.filter(recipient=self.user).active()
        rendered = self.RENDER_TEMPLATE.render(
            Context({'notifications': nf_list}))
        self.assertIn('followed you', rendered)

    def test_render_template_tag_with_empty_notifications(self):
        nf_list = Notification.objects.filter(recipient=self.user).active()
        rendered = self.RENDER_TEMPLATE.render(
            Context({'notifications': nf_list}))
        self.assertIn('No notifications yet', rendered)

    def test_render_template_tag_without_context(self):
        rendered = self.RENDER_TEMPLATE.render(Context({}))
        self.assertIn('No notifications yet', rendered)

    def test_js_inclusion_tag(self):
        rendered = self.JS_INCLUSION.render(Context())
        self.assertIn('<script type="text/javascript">', rendered)

# TODO: Write test for test-worthy methods.



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2015-12-09 07:55
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('actor_object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('actor_text', models.CharField(blank=True, max_length=50, null=True)),
                ('actor_url_text', models.URLField(blank=True, null=True)),
                ('verb', models.CharField(max_length=50)),
                ('description', models.CharField(blank=True, max_length=255, null=True)),
                ('nf_type', models.CharField(default='default', max_length=20)),
                ('target_object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('target_text', models.CharField(blank=True, max_length=50, null=True)),
                ('target_url_text', models.URLField(blank=True, null=True)),
                ('obj_object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('obj_text', models.CharField(blank=True, max_length=50, null=True)),
                ('obj_url_text', models.URLField(blank=True, null=True)),
                ('extra', jsonfield.fields.JSONField(blank=True, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('read', models.BooleanField(default=False)),
                ('deleted', models.BooleanField(default=False)),
                ('actor_content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notify_actor', to='contenttypes.ContentType')),
                ('obj_content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notify_object', to='contenttypes.ContentType')),
                ('recipient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='notifications', to=settings.AUTH_USER_MODEL)),
                ('target_content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notify_target', to='contenttypes.ContentType')),
            ],
            options={
                'ordering': ('-created',),
            },
        ),
    ]



from django import template
from django.core.urlresolvers import reverse
from django.template.loader import render_to_string
from notify import notify_settings

register = template.Library()


class RenderNotificationsNode(template.Node):

    """
    Template node to parse the token supplied and then generate html
    using the template of corresponding notification.
    """

    @classmethod
    def handle_token(cls, parser, token):
        tokens = token.split_contents()

        if tokens[1] != 'using':
            raise template.TemplateSyntaxError(
                "The second argument in %r must be 'for'" % (tokens[0]))

        if len(tokens) == 3:
            return cls(parser.compile_filter(tokens[2]))
        elif len(tokens) == 5:
            if tokens[3] != 'for':
                raise template.TemplateSyntaxError(
                    "The second argument in %r must be 'for'" % (tokens[0]))
            return cls(obj=parser.compile_filter(tokens[2]), target=tokens[4])
        else:
            raise template.TemplateSyntaxError(
                "%r takes 2 or 3 arguments, %r given." % len(tokens))

    def __init__(self, obj, target='page'):
        self.obj = obj
        self.target = target

    # @staticmethod
    def generate_html(self, notifications):
        """
        Generates rendered HTML content using supplied notifications.
        :param notifications: Notification QuerySet Object
        :return: Rendered HTML.
        """
        html_chunks = []
        template_dir = 'notifications/includes/'
        for notification in notifications:

            template_name = notification.nf_type

            if self.target == 'box':
                suffix = '_box.html'
                nf_ctx = notification.as_json()
            else:
                suffix = '.html'
                nf_ctx = {'notification': notification}

            templates = [
                "{0}{1}{2}".format(template_dir, template_name, suffix),
                "{0}default{1}".format(template_dir, suffix)]

            html = render_to_string(templates, nf_ctx)

            html_chunks.append(html)
        else:
            html_chunks.append("<b>No notifications yet.</b>")
        html_string = '\n'.join(html_chunks)
        return html_string

    def render(self, context):
        """
        Render method of the template tag, returns generated html content to
        the parent template where it was called.
        :param context: Template context.
        :return: Generated HTML content using notification queryset object.
        """
        notifications = self.obj.resolve(context)
        return self.generate_html(notifications)


@register.tag
def render_notifications(parser, token):
    """
    Example::
        {% render_notifications for NOTIFICATION_QUERYSET_OBJECT %}

    :param parser: default arg
    :param token: default arg
    :return: Rendered HTML content for supplied notification QuerySet.
    """
    return RenderNotificationsNode.handle_token(parser, token)


@register.inclusion_tag('notifications/includes/js_variables.html')
def include_notify_js_variables():
    """
    Inclusion template tag to include all JS variables required by the
    notify.js file on the HTML page around <script> tags.

    Example::
        {% include_notify_js_variables %}

    :return: Prepares context for rendering in the inclusion file.
    """
    ctx = {
        'update_notification': reverse('notifications:update'),
        'mark_notification': reverse('notifications:mark'),
        'mark_all_notification': reverse('notifications:mark_all'),
        'delete_notification': reverse('notifications:delete'),

        'nf_list_class_selector': notify_settings.NF_LIST_CLASS_SELECTOR,
        'nf_class_selector': notify_settings.SINGLE_NF_CLASS_SELECTOR,
        'nf_box_list_class_selector': notify_settings.NF_BOX_CLASS_SELECTOR,
        'nf_box_class_selector': notify_settings.SINGLE_NF_BOX_CLASS_SELECTOR,

        'nf_mark_selector': notify_settings.MARK_NF_CLASS_SELECTOR,
        'nf_mark_all_selector': notify_settings.MARK_ALL_NF_CLASS_SELECTOR,
        'nf_delete_selector': notify_settings.DELETE_NF_CLASS_SELECTOR,

        'nf_read_class': notify_settings.READ_NF_CLASS,
        'nf_unread_class': notify_settings.UNREAD_NF_CLASS,

        'nf_update_time_interval': notify_settings.UPDATE_TIME_INTERVAL,
    }
    return ctx
